<?php

/*

 $Id: range.php,v 1.63.2.2 2004/01/02 21:44:24 vollmerk Exp $
 $Source: /data/cvsroot/maintain/docs/range.php,v $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/

/*!
	@header Range Document
	Create/edit/delete IP ranges.

*/

/* Start House Keeping */

	require("../modules/config.php");

	// make sure they are supposed to be here
        if (!check_session()) { logout(); }
	
	setup_user_environment($action, $zone_id);
/* End House Keeping */

// Only admins and zone admins should be here
if ( ! $user->is_admin && ! $user->is_zone_admin && ! $action == "display_range_comment") {
        header("Location: " .  $conf['web_path'] . "/index.php");
	exit();	
}

// process any submits that may come our way
$title = "IP Range Information";
$conf['script'] = 'range.php';
// need this object for doing the IP math
$ip = new IP (0);

// if we need it, we need it
if ( $range_id ) {
	$range = new Range($range_id);
}

show_template('header');

?> <p align="left" class="header1"><?= $title ?></p> <?

/***************************
        WORK IF BLOCK 
***************************/


/*
	Do some generic checks for update and create (Doesn't do any specific work)
*/

if ( $action == 'Create' || $action == 'Update' ) {

	// If they aren't an admin they shouldn't be here!
	if (!$user->is_admin) { 
		$conf['ui']->show_access_denied(); 
		exit();
	}

	// if they are changing the range type away from dynamic make sure that it fits
	if ($range->type != $type AND ( $range->type == 'dynamic' OR $range->type == 'ddns' ) ) {

		// Get the hosts first
		$hosts = $range->get_range_hosts();

		// Gonna cheat here.
		$old_type = $range->type;
		$range->type = 'static';

		// Get the free addresses assuming it's a static range
		$count = $range->get_free_address_count();
		
		if ($count < count($hosts) ) { 
			$conf['error']->add_error('range_type',"Not enough room to make this a static range.");
		}

		// Put it back
		$range->type = $old_type;

	} //if type change

	// check the ip field for validity (\d+.\d+.\d+.\d+)
	if (!$ip->is_dotted_quad($ip_start) ) {
		$conf['error']->add_error('range_ip_start', "Please enter a valid IP address.");
	}

	if (!$ip->is_dotted_quad($ip_end) ) {
		$conf['error']->add_error('range_ip_end', "Please enter a valid IP address.");
	}

	$ip_start_int = $ip->ip2int($ip_start);
	$ip_end_int   = $ip->ip2int($ip_end);

	//Check that the range fits inside the picked subnet
	if ($subnet_id) {
		//Get the info
		$subnet = new Subnet($subnet_id);
		$netmask_int = $subnet->netmask->ip_int;
		$subnet_int = $subnet->subnet->ip_int;

		$subnet_end = $subnet->get_end($subnet_int,$netmask_int);

		//Make sure the end is less than the the subnet_end
		if ($ip_end_int > $subnet_end) {
			$conf['error']->add_error('range_ip_start',"Range does not fit inside Subnet");

		} //check end

	} //if subnet fits


	// let's make sure that there is a subnet for this range
        $ip_start_int = $ip->ip2int($ip_start);

	$range = new Range(0);
	$subnet_id = $range->get_subnet($ip_start_int);

	if (!$subnet_id) { 
		$conf['error']->add_error('range_ip_start',"There was no subnet found for this IP Range");
	}

	// make sure start is less than end and its available for this install
	//   of Maintain (the IP's exist in the ip table)


	// start can't be more than end
	if ( $ip_start_int > $ip_end_int ) {
		$conf['error']->add_error('range_ip_start', "The IP start has to be greater than the IP end.");
	}

	// now check that each IP exists in the database
	$sql = "SELECT * FROM ip_blocks WHERE start<='$ip_start_int' AND end>='$ip_end_int'";
	$db_inside_result = mysql_query($sql."'$ip_start_int'", $conf['dbh']);

	if ( ! mysql_fetch_object($db_inside_result) ) {
		$conf['error']->add_error('range_ip_start', "This IP Range it outside your assigned, and enabled, IP Blocks.");
		$conf['error']->add_error('range_ip_end', "This IP Range it outside your assigned, and enabled, IP Blocks.");
	}

	// find out anything that overlaps with the start ip
	$sql = "SELECT * FROM ranges WHERE start<=$ip_start_int AND end>=$ip_start_int AND id!='$range_id'";
	$db_results = mysql_query($sql, $conf['dbh']);

	$start_failed = 0;

	while ( $o = mysql_fetch_object($db_results) ) {
		$start_failed = $o->id;
		$error_msg = "Sorry, but the end IP that you specified overlaps with another range. ".
		"<a href=\"" . $conf['web_path'] . "/range.php?action=edit&range_id=$start_failed\"[ View Overlapping Range ]</a>";
		$conf['error']->add_error('range_top', $error_msg);
	}

	// find anything that overlaps with the ending ip
	$sql = "SELECT * FROM ranges WHERE start<=$ip_end_int AND end>=$ip_end_int AND id!='$range_id'";
	$db_results = mysql_query($sql, $conf['dbh']);


	while ( $o = mysql_fetch_object($db_results) ) {
		$end_failed = $o->id;
		$error_msg = "Sorry, but the start IP that you specified overlaps with another range. ".
			"<a href=\"" . $conf['web_path'] . "/range.php?action=edit&range_id=$end_failed\">[ View Overlapping Range ]</a>";
		$conf['error']->add_error('range_top', $error_msg);
	}

} //end checks

// now do things based on the form data
if ( $action == 'Create' || $action == 'Update' ) {

	// If they aren't an admin they shouldn't be here
	if (!$user->is_admin) { 
		$conf['ui']->show_access_denied();
		exit();
	}

	if ( $conf['error']->error_state ) {
		// there was an error so set the appropriate action
		if ( $action == 'Create' ) {
			$action = 'show_create';
		}
		else {
			$action = 'edit';
			$range = new Range($range_id);
		}
		
		$range->name		  = $name;
		$range->start->ip_addr    = $ip_start;
		$range->end->ip_addr      = $ip_end;
		$range->type              = $type;
		$range->comment           = $comment;

	}

	//no error has occured we need to actualy do the work..
	else {
		$ip_start = $ip->ip2int($ip_start);
		$ip_end   = $ip->ip2int($ip_end);

		// let's clean up the comment field
		$comment = addslashes($comment);

		// create/update the stinkin' range we will need to generate the subnet for this one.
		if ( $action == 'Create' ) {

			$range = new Range(0);
			$subnet_id = $range->get_subnet($ip_start);
			

			//Now go ahead and make this new spiffy ip range
			$sql = "INSERT INTO ranges (start,end,subnet,type,comment,enabled) ".
				"VALUES ('$ip_start','$ip_end', '$subnet_id', '$type', '$comment','true')";

			$db_result = mysql_query($sql, $conf['dbh']);	
			$range_id  = mysql_insert_id($conf['dbh']);
			$range = new Range($range_id);

		}
		//if we aren't creating we must be updating!
		else {

			//Setup the Range var
			$range = new Range($range_id);

			$range->update_enabled($enabled);
			$range->update_comment($comment);
			if ($type != $range->type) { $range->update_type($type); }
			$range->update_end($ip_end);
			$range->update_start($ip_start);
			$range->update_subnet($subnet_id);


		} //end update


		$action = 'edit';
		$range = new Range($range_id);

	} //end if no error


} //end if update/create

// Add a Range --> Zone Mapping. 
// This is what allows a specific zone to use said range
elseif ($action == "add_zone_range") {

	// if they aren't an admin they shouldn't be here
	if (!$user->is_admin) { 
		$conf['ui']->show_access_denied();
		exit();
	} 

	//First check to see if that mapping is there already
	$sql = "SELECT * FROM zone_range WHERE zone='$zone_id' AND range='$range_id'";
	$db_results = mysql_query($sql, $conf['dbh']);

	$r = mysql_fetch_object($db_results);

	if ($r->default_domain == $domain_id AND $r) {
		//We've found it don't need to duplicate, error danger danger will robinson!
		$conf['error']->add_error('zone_range_top',"Error This Zone to Range mapping was already found");
	} 


	elseif ($r->default_domain != $domain_id AND $r) {

		$only_update_domain = true;

	} 
		

	//check to make sure the domain is allow in said zone
	$temp_zone = new Zone($zone_id);
	$domain = new Domain($domain_id);

	if (! $temp_zone->has_domain_access($domain_id) ) { 
		//Nope No Domain for you
		
		$conf['error']->add_error('zone_range_top',"Error $domain->name not allowed in $temp_zone->name");
	} //if no access to domain

	// if want to worry about source domains check that as well
	if ($check_source_domain) {

		$domains = $range->get_represented_domains();

		$sql = "SELECT count(*) FROM zone_domain WHERE zone='$zone_id'";
		foreach ($domains as $domain) {
			$sql .= " AND domain = '" . $domain->id . "'";
		} 
		$db_results = mysql_query($sql, $conf['dbh']);
		
		$count = mysql_fetch_field($db_results);
		
		// if the above returns some results
		if ($count != count($domains)) {
			$conf['error']->add_error('zone_range_top',"Error the Range Domains don't match $temp_zone->name");
		} // if we've found anything

	} // check source domain

	//no errors we are good...
	if (! $conf['error']->error_state ) { 
		
		//let's go ahead and add the zone to range mapping
		if (!$only_update_domain) { $range->add_zone($zone_id,$domain_id); }
		elseif ($only_update_domain) { $range->update_default_domain($zone_id,$domain_id); }

		$conf['ui']->display_confirmation("Zone Mapping Added","(" . $range->start->ip_addr . " - " . $range->end->ip_addr .") has been added as a valid range for $temp_zone->name",$conf['web_path'] . "/range.php?action=edit&amp;range_id=$range_id"); 

	} //if no errors

	else {
		//set the action 
		$action = 'edit';
	}


} //add_zone_range

//delete zone_range mapping
elseif ($action == "delete_zone_range") {

	// If they aren't an admin they shouldn't be here
	if (!$user->is_admin) { 
		$conf['ui']->show_access_denied();
		exit();
	}

	$range = new Range($range_id);

	//Delete the Range
	$range->delete_zone_range($zone_id);

	$action = 'edit';

} //delete_zone_range


//enable/disable a range_zone mapping
elseif ($action == "enable_zone_range") { 

	// If they aren't an admin they shouldn't be here
	if (!$user->is_admin) { 
		$conf['ui']->show_access_denied();
		exit();
	} 

	$sql = "UPDATE zone_range SET enabled='$enabled' WHERE range='$range_id' AND zone='$zone_id'";
	$db_results = mysql_query($sql, $conf['dbh']);

	$action = 'edit';
} //enable_zone_range

/*!
        @function batch_copy
        @discussion much like the batch move, copies all hosts
                    from range X to range Y, setting a new domain
                    and assuming range Y is in a different subnet
*/
elseif ($action == 'batch_copy') {

	// If they aren't an admin they shouldn't be here
	if (!$user->is_admin) { 
		$conf['ui']->show_access_denied();
		exit();
	}

        $src_range_obj = new Range($src_range);
        $dst_range_obj = new Range($dst_range);
        $zone = new Zone($zone_id);

        // Get the hosts in this range
        $hosts = $src_range_obj->get_range_hosts();

	foreach ($hosts as $test_host) { 
		$test_host->domain = new Domain($domain_id);
		$test_hostname = $test_host->name . "." . $test_host->domain->name;
		if ($dst_range_obj->type == 'dynamic' || $dst_range_obj->type == 'ddns') { $test_host->dynamic_range = $dst_range_obj->id; }

		// Test the hostname
		$test_host->check_hostname($test_host->name,$test_host->domain,$test_host->id);
		if ($conf['error']->get_status('name')) {
			$conf['error']->add_error('dst_range',$test_hostname . " already found in destination range");
		}

		// Test the HW
		$test_host->check_hardware_address($test_host->ha,$test_host->id,$dst_range);

	} // foreach

        // First make sure they aren't the one and same range
        if ($dst_range == $src_range) {
                $conf['error']->add_error('dst_range',"You picked the same source and destination ranges!");
        }

        // Now check to make sure there is enought room
        if ($src_range_obj->get_used_ips() > $dst_range_obj->get_free_address_count() and $dst_range_obj->type != "dynamic") {
                $conf['error']->add_error('dst_range','Not Enough Free Addresses');
        }

        // Make sure they aren't in the same subnet
        if ($src_range_obj->subnet == $dst_range_obj->subnet) {
                $conf['error']->add_error('dst_range',"Ranges are in the same Subnet unable to copy");
        }

        // Make sure that the dest domain doesn't exist in the source range
        if ($src_range_obj->has_domain($domain_id)) {
                $conf['error']->add_error('src_range', "Source range contains target Domain");
        }

	if (!$conf['error']->error_state) { 
	
		// We have the hosts from above
	        // Recurse through them and make the copies
	        foreach ($hosts as $host) {

	                if ($dst_range_obj->type == 'dyanmic' OR $dst_range_obj->type == 'ddns') {
	                        $dyanmic_range = $dst_range;
	                }
	                else {
	                        $ip = $dst_range_obj->get_free_ip();
        	        }

	                $zone->create_host($host->name,$domain_id,$ip->ip_int,$dynamic_range,$host->ha,$host->os,$host->type,$host->location,$host->serial,
	                                   $host->other_id,$dynamic_range,$host->workgroup->id,$user->username,$host->purchase_date,$host->po_number,$host->warranty_date,$host->owning_unit,
					   $host->user_id,$host->expire,$host->ttl);

	                unset($ip,$dynamic_range);

        	} // end foreach host

		$conf['ui']->display_confirmation("Hosts Copied", "All hosts have been copied.",$conf['web_path'] . "/admin.php");

	} // no errors

	else {
		$action = 'show_range_host_copy';
	} // else errors


} // batch_copy



/**************************
   DISPLAY IF BLOCK 
**************************/

/*!
        @function show_range_host_copy
        @discussion copies the host in range X over to
                    range Y, Sets a new domain name and
                    requires range Y to be in a different
                    subnet
*/
if ( $action == "show_range_host_copy") {

        require ($conf['prefix'] . "/templates/show_batch_host_copy.inc");

} // show_range_host_copy

/*!
        @function confirm_range_host_copy
        @discussion confirm that they really want to copy these ranges
                    also do the error checking
*/
elseif ($action == "confirm_range_host_copy" AND $user->is_admin) {

        //Create objects
        $src_range_obj = new Range($src_range);
        $dst_range_obj = new Range($dst_range);

        // First make sure they aren't the one and same range
        if ($dst_range == $src_range) {
                $conf['error']->add_error('dst_range',"You picked the same source and destination ranges!");
        }

        // Now check to make sure there is enought room
        if ($src_range_obj->get_used_ips() > $dst_range_obj->get_free_address_count() and $dst_range_obj->type != "dynamic") {
                $conf['error']->add_error('dst_range','Not Enough Free Addresses');
        }

        // Make sure they aren't in the same subnet
        if ($src_range_obj->subnet == $dst_range_obj->subnet) {
                $conf['error']->add_error('dst_range',"Ranges are in the same Subnet unable to copy");
        }

        // Make sure that the dest domain doesn't exist in the source range
        if ($src_range_obj->has_domain($domain_id)) {
                $conf['error']->add_error('src_range', "Source range contains target Domain");
        }

        //If no error
        if (!$conf['error']->error_state) {

                $conf['ui']->show_confirm($conf['web_path'] . "/range.php?action=batch_copy&src_range=$src_range&dst_range=$dst_range&domain_id=$domain_id&zone_id=$zone_id",
                                         "Are you sure you want to move copy all hosts from [ " . $src_range_obj->start->ip_addr . " - " .
                                         $src_range_obj->end->ip_addr . " ] to [ " . $dst_range_obj->start->ip_addr . " - " . $dst_range_obj->end->ip_addr .
                                         " ] ");

        } // no error
        else {
                require ($conf['prefix'] . "/templates/show_batch_host_copy.inc");
        }

} // confirm_range_host_copy


if ( $action == "edit_range_comment") { 
	$range = new Range($range_id); 
	
	require($conf['prefix'] . "/templates/show_range_comment.inc");

} //edit_range_comment

//show the zone_range mapping
elseif ( $action == "show_zone_range_mapping") {

	$ranges = $conf['zone']->ip_ranges();
	if (!$zone) { $zone = $conf['zone']; }
	require($conf['prefix'] . "/templates/show_zone_ranges.inc");

}

//confirm range_zone mapping deletion
elseif ( $action == "confirm_delete_zone_range") { 
	$zone = new Zone($zone_id);
	$range = new Range($range_id);

	if (count($range->zones) > 1 OR $range->enabled == "false" ) { 
		$conf['ui']->show_confirm($conf['web_path'] . "/range.php?action=delete_zone_range&zone_id=$zone_id&range_id=$range_id","Are you sure you want to remove this range from $zone->name");
	}
	// else there is only one range left
	else {
		$conf['ui']->display_confirmation("Error","Unable to remove last Zone --> Range Mapping, all ranges must be assoicated with at least one zone if they are enabled", $conf['web_path'] . "/range.php?action=edit&range_id=$range_id");
	}

}

//update zone_range comment
elseif ( $action == "update_comment") { 
	$range = new Range($range_id);

	$range->update_zone_comment($comment);
	$ranges[] = $range;
	$action = "display_range_comment";
	

} //end update_comment

//Check for Split Range Mojo
elseif ( $action == 'Split Range' ) {
	
	// They must be an admin
	if (!$user->is_admin) { 
		$conf['ui']->show_access_denied();
		exit();
	}


	// check if ip_end_1 < ip_start_2
	if ( $ip_end_1 >= $ip_start_2) {
		$conf['error']->add_error('range_top', "The first IP end must be less than the second IP start.");
		$action = 'show_split';
	}
	
	//check to make sure they havn't refreshed and we aren't respliting the same range again.
	$sql = "SELECT * FROM ranges WHERE start='$ip_start_2' AND end='$ip_end_2'";
	$db_results = mysql_query($sql, $conf['dbh']);
	$r = mysql_fetch_object($db_results);
	if ($r) {
		$conf['error']->add_error('range_top', "The second range was already found, this can happen if you have attempted to refresh the page, or the submit button was hit twice. The new range has now been selected");
		$range = new Range($r->id);
		$action = 'show_split';
	}
		
	//if we didn't find any errors then go ahead and split the range
	if (! $conf['error']->error_state) {

		$range = new Range($range_id);

		//update the old range with start_1 -> end_1
		$sql = "UPDATE ranges SET start=$ip_start_1, end=$ip_end_1 WHERE id=$range_id";
		mysql_query($sql, $conf['dbh']);

		// insert the new zone_range
		$sql = "INSERT INTO ranges (start,end,type,subnet,enabled) VALUES ".
			"('$ip_start_2', '$ip_end_2', '$range->type', '$range->subnet', '$range->enabled')";
		mysql_query($sql, $conf['dbh']);
		$range_id_2 = mysql_insert_id($conf['dbh']);

	//add this new range to the zones that the old one is in
	$range_2_obj = new Range($range_id_2);

	foreach ($range->zones as $zone_id) {
		$range_2_obj->add_zone($zone_id,$range->get_default_domain($zone_id));
	}
	
	unset($user->view);
	$action = 'edit';

	
	}
}
//ok were merging the ranges
elseif ($action == "merge_ranges" and $merge_range) {

	// They must be an admin
	if (!$user->is_admin) { 
		$conf['ui']->show_access_denied();
		exit();
	}

	$src_range = new Range($range_id);
	$dst_range = new Range($merge_range);

	//check to see which end we are attaching it to. and modify zone_range
	if ($src_range->end->ip_int+1 == $dst_range->start->ip_int) {
		$src_range->append_range($merge_range);
	} //append dst range
	elseif ($src_range->start->ip_int-1 == $dst_range->end->ip_int) {
		$src_range->prepend_range($merge_range);
	} //prepend dst range
	
	//reset the range object to account for the new start and end
	$range = new Range($range_id);

	//update any zone_range references
	$range->replace_zone_range($range_id,$merge_range);

	//update the dynamic_range if needed
	$range->replace_dynamic_range($range_id,$merge_range);


	unset($user->view);
	$action = 'edit';

} //merge_range

elseif ($action == "merge_ranges" and !($merge_range) ) {
	$action = "show_zone_ranges";
} //we were trying to merge a range but it didn't work out...	

// this we're really deleting this zone
elseif ( $action == 'delete' && $really ) {

	$range = new Range($range_id);

	$range->delete_self();

	$conf['ui']->display_confirmation("Range Deleted","(" . $range->start->ip_addr . " - " . $range->end->ip_addr . ")","/index.php");
	unset($range,$action);

	}

/*!
	@action Confirm Batch Move
	@discussion we are confirming a batch move of hosts from src_range to dst_range
*/
elseif ($action == 'confirm_batch_move') {

        $src_range_obj = new Range($src_range); 
        $dst_range_obj = new Range($dst_range);

        //first let's check to make sure they aren't trying to move from A -> A
        if ($src_range == $dst_range) {
                $conf['error']->add_error('dst_range','Destination can\'t be the same as Source');
        }

        //now check to make sure there is room!
        if ($src_range_obj->get_used_ips() > $dst_range_obj->get_free_address_count() and $dst_range_obj->type != "dynamic") {
                $conf['error']->add_error('dst_range','Not Enough Free Addresses');
        }

	
	/*
	  check to see if we are duplicating any mac addresses
	*/

	//get all the dst range siblings (all ranges in the same subnet)
	$dst_ranges = $dst_range_obj->get_sibling_ranges();

	//get the hosts in the source range
	$src_hosts = $src_range_obj->get_range_hosts();

	//recurse through all the hosts!
	foreach ($src_hosts as $src_host) {

		$sql = "SELECT * FROM host WHERE ha='$src_host->ha' AND (";

		//add dynamic_range = '' or ip >= '' and <= '' to sql statement
		foreach ($dst_ranges as $temp_range) {
			if ($temp_range->type == 'static') {
				if (substr_count($sql, "ip") OR substr_count($sql,"range") ) { $sql .= " OR ( ip<='". $temp_range->end->ip_int ."' AND ip>='" . $temp_range->start->ip_int ."' ) "; }
				else { $sql .= " ( ip<='" . $temp_range->end->ip_int ."' AND ip>='" . $temp_range->start->ip_int ."' ) "; }
			}
			elseif ($temp_range->type == 'dynamic' OR $temp_range->type == 'ddns') {
				if (substr_count($sql, "ip") OR substr_count($sql,"range") ) { $sql .= " OR dynamic_range='" . $temp_range->id ."' "; }
				else { $sql .= " dynamic_range='" . $temp_range->id ."' "; }				
			} 

		} //recurse through ranges

		//exclude ha's with 000000000000
		$sql .= " ) AND ha!='000000000000'";

	$db_results = mysql_query($sql, $conf['dbh']);
	
	if ($r = mysql_fetch_row($db_results)) {
		$conf['error']->add_error('dst_range',"Duplicate Hardware Address found [" . $src_host->ha ."] (".$src_host->ip->ip_addr.")");
	}
		

	} //recurse through hosts



        //if an error has occured return to the show
        if ($conf['error']->error_state) {
                $action = "show_batch_move";
        }

        //do a show_confirmation that display what is going to happen if they do this!
        else {

	//if it's dynamic and they are going to overfill it warn them!
        if ($dst_range_obj->type == "dynamic" and ($src_range_obj->get_used_ips()+$dst_range_obj->get_used_ips()) > $dst_range_obj->total_ips) {
		$force_update = "<br /><br /><input type=\"checkbox\" name=\"force_move\" value=\"1\" /><font class=\"error\">You are attempting to overfill a dynamic range. Are you sure you want to do this?</font>";
	}


	//do a check to see if matching the IP's is possible
	if ($match_ip) {
		$found = 0;

		//get the hosts in the source range
		$hosts = $src_range_obj->get_range_hosts();

		//get the XXX.XXX.XXX.XXX format of the address
		$src_addr = $src_range_obj->start->ip_addr;
		$dst_addr = $dst_range_obj->start->ip_addr;
		
		//do a preg replace on the addresses to set them to XXX.XXX.XXX.0
		$src_addr = preg_replace("/(\d+\.\d+\.\d+)\.\d+/","\\1.0", $src_addr);
		$dst_addr = preg_replace("/(\d+\.\d+\.\d+)\.\d+/","\\1.0", $dst_addr);

		//convert them to INT's and find the difference. Make a IP object for the math mojo
		$ip = new Ip(0);
		$src_addr = $ip->ip2int($src_addr);
		$dst_addr = $ip->ip2int($dst_addr);
		$addr_dif = $dst_addr-$src_addr;
					
		//go through the hosts and check the "NEW" ip address
		foreach ($hosts as $host) {
			$new_ip = $host->ip->ip_int + $addr_dif;
			if ($dst_range_obj->check_free_ip_address($new_ip)) {
				$found = 1;
			} //end if for is free
		} //end foreach

		//if there was a dst ip found show the "ARE YOU SURE YOU WANT TO BREAK IT" box
		if ($found) { 
			$force_update = "<br /><br /><input type=\"checkbox\" name=\"force_move\" value=\"1\" />";
			$force_update .= "<font class=\"error\">Unable to Match fourth octet to fourth octet during move. Continue?</font>";
		}
		//else just pass the var
		else {
			$force_update = "<input type=\"hidden\" name=\"force_move\" value=\"1\" />";
			$force_update .= "\n<input type=\"hidden\" name=\"addr_dif\" value=\"$addr_dif\" />";
		}
	} //if match ip

	//stick this in there so it works right... 
	else {
		$force_update = "<input type=\"hidden\" name=\"force_move\" value=\"1\" />";
	}
        $src_range_name = "[" . $src_range_obj->start->ip_addr . " - " . $src_range_obj->end->ip_addr . "]";
        $dst_range_name = "[" . $dst_range_obj->start->ip_addr . " - " . $dst_range_obj->end->ip_addr . "]";

        $dst_remaining = $dst_range_obj->total_ips - ($src_range_obj->get_used_ips() + $dst_range_obj->get_used_ips());

	if ($user->invalid_batch_move_hosts) { 
			$force_update .= "<br /><br /><font class=\"error\">The Following Hosts will not be moved due to MAC address conflicts in the Destination range<br /></font>";
			foreach ($user->invalid_batch_move_hosts as $host_id) {
				$host = new Host($host_id);
				$force_update .= "$host->hostname<br />";			
			} //end foreach hosts

	} //end if invalid_batch_move_hosts

	$total_moved = $src_range_obj->get_used_ips();
        $conf['ui']->show_confirm($conf['web_path'] . "/range.php?action=batch_move&src_range=$src_range&dst_range=$dst_range&match_ip=$match_ip",
	"<p>This will move $total_moved hosts from $src_range_name into $dst_range_name." .
        " There will be $dst_remaining remaining free IP's in $dst_range_name." .
        " Are you are sure you want to do this?\n$force_update");
        }

} //confirm_batch_move

/*!
	@action
	@discussion we've confirmed that we going to move all 
	the hosts lets get to it, stop slackin!
*/
elseif ($action == "batch_move" && $force_move) {

	// They must be an admin
	if (!$user->is_admin) { 
		$conf['ui']->show_access_denied();
		exit();
	}

	$src_range_obj = new Range($src_range);
	$dst_range_obj = new Range($dst_range);

	//get the hosts in the source range
	$hosts = $src_range_obj->get_range_hosts();	

	//Update IP of all the hosts
	if ($dst_range_obj->type == "dynamic") {
		foreach ($hosts as $host) {
			
			//set the ip to zero! and setup the dynamic range
			$host->update_ip(0,$dst_range);		

		} //end foreach
	} 
	else {	
		// Go through each host and try to match up
		// Ip addresses, if not just grab a random one
		foreach ($hosts as $host) {

	                $new_ip->ip_int = $host->ip->ip_int + $addr_dif;

			// Check to see if the New IP is free
			if (!$host->ip->is_free($new_ip->ip_int)) {
				$new_ip = $dst_range_obj->get_free_ip_address();
			}

			// Let's go ahead and updated the ip address now
			$host->update_ip($new_ip->ip_int);
	
		} //end foreach
	} //end else


	$conf['ui']->display_confirmation("Hosts Moved", "All Hosts moved to new Range....", $conf['web_path'] . "/admin.php");

} // batch_move

//if they are trying to overfill and they didn't check the box display a note and throw them back to index
elseif ($action == "batch_move" && !$force_move) {
		$conf['ui']->display_confirmation("Host <font class=\"error\">NOT</font> Moved", "Your Hosts were not moved", $conf['web_path'] . "/index.php");
} // show error message for batch move

//show merge ranges
if ($action == "show_merge") {

	$range = new Range($range_id);
	require ($conf['prefix'] . "/templates/show_merge_range.inc");

} // show_merge

// Show batch move
elseif ($action == "show_batch_move") { 
	require ($conf['prefix'] . "/templates/show_batch_host_move.inc");
} // show_batch_move
	
elseif ( $action == 'edit' || $action == 'show_create' ) {

	$range = new Range($range_id);

	// show the range edit form
	$conf['ui']->show_range($action, $range);
}

// Go through and find all unused ranges
elseif ( $action == 'show_unused_ranges' ) {
	$r = get_available_ranges();
	$conf['ui']->show_unused_ranges($r);
} // show_unused ranges

//show a zone specific range
elseif ( $action == 'show_zone_ranges' ) { 
	// just show a list of the currently available IP's
        $user->view = new View();

        if ( $sort_type ) {
	        $user->view->initialize($sort_type, $sort_offset);
        }
        else {
        	$user->view->initialize('start', 0);
        }

	$ranges = $conf['zone']->ip_ranges($user->view);
	$host = true;
        $conf['ui']->show_ip_ranges($host, $ranges);
} // show_zone_ranges

//shows all the possible ranges (admin tool)
elseif ( $action == 'show_ranges' && $user->is_admin) {
		
	$sql = "SELECT * FROM ranges";
	if (!$user->view) { 
                $user->view = new View($sql);
	}

	if ($sort_type) {
	       $user->view->initialize($sort_type,$sort_offset);
	}
        else {
        	$user->view->initialize('start', $sort_offset);
        }

	$ranges = $conf['ui']->get_ranges($user->view->get_query());
	$total_items = count_ranges();	

	$conf['ui']->show_ranges($ranges,$total_items);
} //show_ranges

elseif ( $action == 'show_split' ) {
	// show the split range form	
	$range = new Range($range_id);
	$conf['ui']->show_split_range($range);
} //show_split

elseif ( $action == 'delete' || $action == 'Delete' ) {
	$range = new Range($range_id);

	//for gosh sakes man think of the children!
	$child = new Child("range", $range->id,0,$range);
		if ($child->has_children) {
			$conf['ui']->display_confirmation("<font class=\"fatalerror\">Fatal Error, $child->total child objects found", "Please remove the below children and try again", $conf['web_path'] . "/range.php?action=show_ranges"); 
			$child->show_children();
			}
		else {
			// show the confirm delete
			$zone  = new Zone($range->zone);
			$start = $range->start->ip_addr;
			$end   = $range->end->ip_addr;
			$conf['ui']->show_confirm($conf['web_path'] . "/range.php?action=delete&really=1&range_id=$range_id","Are you sure you want to delete this range? <b>($start to $end)</b>");
		}
} //delete 

// if we want to see ranges for just a few zones
elseif ( $action == 'show_zone_ranges' && $zone_id ) {
	if (!$user->view) {
		$user->view = new View("SELECT ranges.id FROM ranges,zone_range WHERE ranges.id = zone_range.range AND zone_range.zone=$zone_id");
	}

	if ( $sort_type ) {
		$user->view->initialize($sort_type,$sort_offset);
	}
	else {
		$user->view->initialize('start', $sort_offset);
	}
	
	//retrieve the ranges based on the view.
	$ranges     = $conf['ui']->get_ranges();
	$conf['ui']->show_ip_ranges('', $ranges);
} //show_zone_ranges

?>

<? show_template('footer'); ?>
