<?php
/*

 $Id: search.php,v 1.58.2.2 2004/01/16 06:28:26 vollmerk Exp $
 $Source: /data/cvsroot/maintain/docs/search.php,v $

 Copyright (c) 2002 Oregon State University - Network Engineering
 All rights reserved.

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

*/

/*!
	@header Search Document
	This document handles all the searching features of maintain, depends upon the search class.
*/

/* Start Housekeeping */
	require "../modules/config.php";

        if (!check_session()) { logout(); }
	
	setup_user_environment($action, $zone_id);

	// trim the string, just incase
	$search_string = trim($search_string);

	if (!$search_type) { $search_type = 'fuzzy'; }
	// stuff we need to save in conf for the search settings
	// it's a cheep fix the class does no more than hold the vars for now.
	$conf['search'] = new Search($search_form_type,$search_string,$search_field,$search_type,$search_zone,$start_range,$end_range);
	$conf['search']->override_limit = $override_limit;
	// $conf['action'] .= "&search_field=" . $search_field . "&search_string" . $search_string;
	$conf['search_action'] = "search.php?action=search_hosts&search_field=$search_field&search_string=$search_string&search_form_type=$search_form_type&override_limit=$override_limit";
	$conf['script'] = "search.php";
	
	// needed objects
	$ip = new IP(0);

	if ($view_reset) { $user->view = false; }


	if (preg_match("/^\d{1,3}\.\d{1,3}\.\d{1,3}$/", $search_string) && $search_field =='ip') {
		$start_range = $search_string . ".0";
		$end_range   = $search_string . ".255";
		$search_field = "fuzzy_ip";
		$search_string = 'fuzzy_ip';
	} // End if it's 3/4 a dotted quad
	

/* End Housekeeping */

/*!
	@function search_hosts
	@discussion this is called from the search template, checks the search
*/
if ($action == "search_hosts") { 

	// Check Non-Fuzzy Hardware addresses
	if ($search_field == 'ha' && $search_type != 'fuzzy') { 
		// check the search string, cleans and checks the hardware address.
		// returns the cleaned hardware address 
		if (!$mac = $conf['ui']->is_hardware_address($search_string)) { 
			$conf['error']->add_error('search_string',"Invalid Hardware Address Entered. Must be 12chr (A-F 0-9)");
		}
		if ($mac) { $search_string = $mac; }

	} // search_field == ha

	// Check Fuzzy Hardware Addresses
	if ($search_field == 'ha' && $search_type == 'fuzzy') {
		$search_string = $conf['ui']->clean_hardware_address($search_string);
		if (!preg_match("/^[0-9a-fA-F]+/", $search_string)) {
			$conf['error']->add_error('search_string', "Invalid Hardware Address Entered. Must be (A-F 0-9)");
		}

	} // search_field == ha AND fuzzy


	// do checking for a ip search 
	elseif ($search_field == 'ip') { 

		// make sure it's in dotted quad 
		if (!$ip->is_dotted_quad($search_string)) {
			$conf['error']->add_error('search_string', "Invalid IP Address Entered");
		}

		// make sure the ip is inside a ipblock the is enabled
		$ip_int = $ip->ip2int($search_string);
		if (!$ip->ip_exists($ip_int)) {
			$conf['error']->add_error('search_string', "Specified IP doesn't exist");
		}

	} // search_field == ip

	// do checking for range_search
	elseif ($search_field == "range_search") {
		$start_int = $ip->ip2int($start_range);
		$end_int = $ip->ip2int($end_range);

		// check to make sure they are valid ips
		if (!$ip->is_dotted_quad($start_range) OR !$ip->ip_exists($start_int)) {
			$conf['error']->add_error('start_range',"Invalid Start Range");
		}
		if (!$ip->is_dotted_quad($end_range) OR !$ip->ip_exists($end_int)) {
			$conf['error']->add_error('end_range',"Invalid End Range");
		}

	} // search_field == range_search

	// do checking for fuzzy ip search
	elseif ($search_field == "fuzzy_ip") {

		// check to make sure they are valid ips
		if (!$ip->is_dotted_quad($start_range) ) {
			$conf['error']->add_error('start_range', "Invalid Start Range");
		} // start is valid

		if (!$ip->is_dotted_quad($end_range) ) {
			$conf['error']->add_error('end_range', "Invalid End Range");
		} // end is valid

		$start_int = $ip->ip2int($start_range);
		$end_int = $ip->ip2int($end_range);

		// check to make sure start before end
		if ($start_int >= $end_int) {
			$conf['error']->add_error('end_range', "End range is before start range");
		} // start < end 

		// check to make sure both ips are in the ip blocks
		if (!$ip->ip_exists($start_int)) {
			$conf['error']->add_error('start_range',"Specified IP doesn't exist");
		}
		if (!$ip->ip_exists($end_int)) {
			$conf['error']->add_error('end_range',"Specified IP doesn't exist");
		}

	} //checking for fuzzy_ip

	// this can be a hardware address a host name or a ip address so we will need to figure it out and
	// then change the search as needed
	elseif ($search_field == "host_history") { 
		
		$ip_int = $ip->ip2int($search_string);

		if ($ip->is_dotted_quad($search_string) AND $ip->ip_exists($ip_int)) {
			$search_field = "host_history_ip";
			$conf['search']->no_search = true;
		}

		// if 00:00:00:00:00:00 or 000000000000 it's a hardware address 
		// or any variant of that...
		elseif ($conf['ui']->is_hardware_address($search_string)) {
			$search_field = "host_history_ha";
			$conf['search']->no_search = true;
		}


	} // if host_history


	// lets check to see if they have tried to put in the fully qualified domain name...
	elseif ($search_field == "name" || $search_field == "cname" || $search_field == "mx") {

		// if they've put a period for .whee.com
		if (preg_match("/\./",$search_string) ) {

			// check to see if their "domain" is around after spliting it out
			$data = split_hostname($search_string);
			$sql = "SELECT id,name FROM domain WHERE name='$data->domain'";
			$db_results = mysql_query($sql, $conf['dbh']);

			// ok we found the domain switch the search type			
			if (mysql_fetch_row($db_results)) {
				if ($search_field == "name") { $search_field = "fqd"; }
				elseif ($search_field == "cname") { $search_field = "fqd_cname"; }
				elseif ($search_field == "mx") { $search_field = "fqd_mx"; }
			} // if we found it then

		} // if there is a period in the search string
	} // search_field == name | cname | mx


	// let's make sure they entered a search....
	if ($search_string <= " " and !$user->view) { 
		$conf['error']->add_error('search_string',"No Search Entered");
	} // if no search

} // end checking 

/*!
	@function search_hosts2
	@discussion This is the second part of searching, it does the actual work. Check error_state
*/

if ($action == "search_hosts" AND !$conf['error']->error_state AND !$conf['search']->no_search) {

	// if we havn't passed a search string, we are returning with a view
	if ($user->view) {	
		$sql = $user->view->get_query();
		$results = $conf['search']->search_object($search_field,0,0,0,$sql);
	}

	// if we aren't doing a fuzzy ip search
	elseif ($search_string != "fuzzy_ip" AND $search_string != "range_search") 
	{	
		$user->view = false;
		//This is a temp thing.. We need to find a way to do a fuzzy IP search.... 
		if ($search_type == "fuzzy" and $search_field == "ip") {
			$search_type = "exact";
		} //check for fuzzy + ip
		if ($search_type == "fuzzy" and (substr_count($search_string, "%")) == "0") { 
			$search_string .= "%";
			$search_string = substr_replace($search_string,"%",0,0); 
		}	

		$search_string = trim($search_string);
		$results = $conf['search']->search_object($search_field, $search_zone, $search_string, $search_type); 

	} // if normal search

	// this happens if we do a fuzzy ip or range search
	else {
		// clear some vars
		$user->view = false;
		unset($search_string);
		$search_string[] = $start_range;
		$search_string[] = $end_range;
		$results = $conf['search']->search_object($search_field,$search_zone,$search_string,$search_type);

	} // else

	// we need to do this every time checks to make sure we aren't going to get to many results
	$sql = $user->view->get_sql();
	$total_items = mysql_num_rows(mysql_query($sql, $conf['dbh']));

	if ( !$override_limit AND $total_items >= $conf['max_search_results']) { 
		$conf['error']->add_error('search_string',"Error to many results returned ($total_items Objects found)"); 
		unset($action);
	} //if over the limit
	
} // action == search hosts (number two the work!)

// show the header
show_template('header'); 

// setup the top widget
$new_type = $search_form_type . "_class";
${$new_type} = 'error';
if (!$search_form_type) { $simple_class = 'error'; }

$items = array();
$items[] = "<a href=\"" . $conf['web_path'] . "/search.php\" class=\"$simple_class\">Simple Search</a>";
$items[] = "<a href=\"" . $conf['web_path'] . "/search.php?search_form_type=advanced\" class=\"$advanced_class\">Advanced Search</a>";
$items[] = "<a href=\"" . $conf['web_path'] . "/search.php?search_form_type=fuzzy_ip\" class=\"$fuzzy_ip_class\">Fuzzy IP Search</a>";
$items[] = "<a href=\"" . $conf['web_path'] . "/search.php?search_form_type=range_search\" class=\"$range_search_class\">Range Search</a>";
$conf['ui']->display_top_widget($items); 

// always show this template
show_template('search');

// only do this if no errors, and we have things to display
if ($total_items >= "1"and !$conf['error']->error_state) { 
	if ($search_field == 'cname' OR $search_field == 'cserver' OR $search_field == 'fqd_cname') {
		$type = "CNAME";
	}
	elseif ($search_field == 'mx' OR $search_field == 'mxserver' OR $search_field == 'fqd_mx') {
		$type = "MX";
	}
	elseif ($search_field == 'domain') {
		$type = "Domain";
	}
	elseif ($search_field == 'user') {
		$type = "User";
	}
	elseif ($search_field == 'range_search') {
		$type = "Range";
	}
	elseif ($search_field == 'host_history_ip' || $search_field == 'host_history_ha') { 
		$type = "Host History";
	}
	elseif ($search_field == 'workgroup_name') {
		$type = "Workgroup";
	}
	else {
		$type = "Host";
	}
?>
<p align="left"><font class="header2">Showing <?= $type; ?> Search Results</font><br />
<?
//Generate the numbers so we can show X-Z of total Y results 
        $current_items = $user->view->offset + $user->list_view - 1;
        if ($current_items > $total_items) { $current_items = $total_items; }
	if ($user->view->offset) { 
		$start_items = $user->view->offset; 
	}
	else { 
		$start_items = "0";
	}
		
?>
<?= $total_items . " " . $type . "(s)"; ?> found, showing <?= $start_items; ?> - <?= $current_items; ?></p>
<?
} //end if total_items


/*#######################
 Start the Search Display
#######################*/

// put top level condition to avoid this if there is an error
if ($action == "search_hosts" AND !$conf['error']->error_state) {

	// if cname/cserver/fqd cname search
	if ($search_field == "cname" or $search_field == "cserver" OR $search_field == "fqd_cname" ) {
		if ($total_items > "1") {
			$conf['ui']->show_cnames($results,$total_items);
		}
		elseif ($total_items == "1") {
			$conf['ui']->show_cname($results[0], "read");
			$user->view = false;
		}
	} // search_field == cname/cserver/fqd_cname

	// if mx/mxserver/fqd_mx search
	elseif ($search_field == "mx" or $search_field == "mxserver" OR $search_field == "fqd_mx" ) {
		if ($total_items > "1" ) {
			$conf['ui']->show_mxs($results, $total_items);
		}
		elseif ($total_items == "1") {
			$conf['ui']->show_mx($results[0], "read");
			$user->view = false;
		}
	} // search_field == mx/mxserver/fqd_mx

	// if domain search
	elseif ($search_field == "domain" ) {
		if ($total_items > "1" ) {
			$conf['ui']->show_domains($results,$total_items);
		}
		elseif ($total_items == "1") {
			$conf['ui']->show_domain($results[0], "read");
			$user->view = false;
		}
	} // search_field == domain

	// if it's a workgroup search
	elseif ($search_field == 'workgroup_name') {
		if ($total_items > '1') {
			$conf['ui']->show_workgroups(0,$results);
		}
		elseif ($total_items == '1') {
			$workgroup = array_pop($results);
			$conf['ui']->show_workgroup('edit',$workgroup);
		}
	} // search_field == workgroup_name

	// if range_search
	elseif ($search_field == "range_search" ) {
		if ($total_items > "1" ) {
			$conf['ui']->show_ranges($results,$total_items);
		}
		elseif ($total_items == "1") {
			$conf['ui']->show_ranges($results,1);
			$user->view = false;
		}

	} // search_field == range_search

	// if user search
	elseif ($search_field == "user") {
		if ($total_items > "1" ) {
			$conf['ui']->show_users($results,$total_items); 
		}
		elseif ($total_items == "1") {
			$conf['ui']->show_users($results, 1);
			$user->view = false;
		}
	} // search_field == user

	// if it's a host_history_ha search
	elseif ($search_field == "host_history_ha") { 


		$mac = $conf['ui']->clean_hardware_address($search_string);		
		$conf['ui']->show_host_history(0,0,$mac);

	} // search_field == host_history_ha

	// if it's a host_history_ip search
	elseif ($search_field == "host_history_ip") {
		
		$ip_int = $ip->ip2int($search_string);
		
		$conf['ui']->show_host_history(0,$ip_int,0);

	} // search_field == host_history_ha

	elseif ($search_field == "host_history") {
		
		if ($total_items > "1") {
			$conf['ui']->show_hosts($results, $total_items);
		}
		elseif ($total_items == "1") {
			$conf['ui']->show_host_history($results[0]->id);
		}

	} // search_field == host_history

	// if it's hosts we are after
	else {
		if ($total_items > "1")  { 
			$conf['ui']->show_hosts($results, $total_items);	
		}
		elseif ($total_items == "1") {
			$conf['ui']->show_host($results[0], "read");
			$user->view = false;
		}
	} // search_field == hosts

} //end if no errors and we are searching

// if we didn't find anything!
if ($total_items <= "0" AND $action == "search_hosts" AND !$conf['error']->error_state && !$conf['search']->no_search) { 
	if ($conf['search']->type == "fuzzy") { $search_string = preg_replace("/%/","",$search_string); }
	echo "<p align=\"center\"><font class=\"fatalerror\">$search_string Not found</font></p>"; 
	$user->view = false;
} 

	
show_template('footer'); 
?>
